--weights analysis etc


--------------------------------------------------------------------------------------------------------------------------
function select_weight_date_range()
--------------------------------------------------------------------------------------------------------------------------
sequence result
atom dfj, dtj
sequence dfs, dts

dfs = "01/01/1970"
dts = DateToStr(nowDate())

SetSize(6, 25)
result = MixedForm("Set dates", {BRIGHT_WHITE, BLUE}, floor((SCREEN_LINES/2) - 3), floor((SCREEN_COLS/2) - 12),
                        {       {MF_LABEL, 3,2, "From "},
                                {MF_LABEL, 4,2, "To   "},
                                {MF_TEXT,  3,7, 14, dfs},
                                {MF_TEXT,  4,7, 14, dts},
                                {MF_BUTTON, 5, 7, 7, 1, "OK"}
                        })

result[3] = dateStrtoJdays(result[3])
result[4] = dateStrtoJdays(result[4])

return {result[3], result[4]}
end function



--------------------------------------------------------------------------------------------------------------------------
global function weight_analysis()
--------------------------------------------------------------------------------------------------------------------------
sequence cmd, data, weight_table, tmp, wt_tmp, fname, gname, graph
integer fp, above_count, below_count
atom min_wt, max_wt, net_gain, kgpw, avg_wt, tot_wt, date_from, date_to

atom    days_per_seg,           --number of days for each 'slot'
        days_diff,
        kgs_per_seg             --kgs for each slot
integer gd, gw                  --graph date and graph weight


--select a date range
tmp = select_weight_date_range()
date_from = tmp[1]
date_to = tmp[2]

cmd = sprintf("SELECT * FROM Notes WHERE type LIKE '%s' AND patient_no = %d AND jdate >= %d AND jdate <= %d", {"W%", CurPatient[PA_PATIENT_NO], date_from, date_to} )
data = query_database("xnotes.sql", cmd)

if length(data) = 0 then
        VOID = get_yn_box("No weights recorded")
        return 0
end if

if length(data) = 2 then
        VOID = get_yn_box("Need more than one recording!")
        return 0
end if

--extract a weigh table
weight_table = {}

tmp = data[2..$]
for i = 1 to length(tmp) do
        wt_tmp = {{},{}}
        wt_tmp[1] = strtoval(tmp[i][N_NOTE])
        wt_tmp[2] = strtoval(tmp[i][N_JDATE])
        weight_table = append(weight_table, wt_tmp)
end for

--remove obviously erroneous weights
avg_wt = 0
tot_wt = 0
for i = 1 to length(weight_table) do
        tot_wt += weight_table[i][1] 
end for
avg_wt = tot_wt / length(weight_table)

--now count the number of measurements above and below the avg
above_count = 0
below_count = 0
for i = 1 to length(weight_table) do
        if weight_table[i][1] > avg_wt then
                above_count += 1
        else
                below_count += 1
        end if            
end for

--theoretically above and below count should be roughly equal - although may not be!

--this is rather crude, but it should do for most practical purposes
if above_count = 1 then
        wt_tmp = {}
        for i = 1 to length(weight_table) do
                if weight_table[i][1] <= avg_wt then
                        wt_tmp = append(wt_tmp, weight_table[i])
                end if
        end for
        weight_table = wt_tmp
end if

if below_count = 1 then
        wt_tmp = {}
        for i = 1 to length(weight_table) do
                if weight_table[i][1] >= avg_wt then
                        wt_tmp = append(wt_tmp, weight_table[i])
                end if
        end for
        weight_table = wt_tmp
end if


--create report
--may want to link in to target wt for breed

fname = sprintf("%d%d.wt", {CurClient[CL_CLIENT_NO], CurPatient[PA_PATIENT_NO]})
fp = open(fname, "w")

--create page header
printf(fp, "%s\n", {globalSettings[GS_PRACTICE]})
printf(fp, "%s\n", {globalSettings[GS_ADDRESS1]})
printf(fp, "%s\n", {globalSettings[GS_ADDRESS2]})
printf(fp, "%s\n", {globalSettings[GS_ADDRESS3]})
printf(fp, "%s\n", {globalSettings[GS_ADDRESS4]})
printf(fp, "%s\n", {globalSettings[GS_POSTCODE]})
printf(fp, "%s\n", {globalSettings[GS_TEL]})
printf(fp, "Branch : %s\n\n", {globalSettings[GS_BRANCH]})

printf(fp, "Client : %-30.30s    Patient : %s\n", {remove_trailing_spaces(CurClient[CL_TITLE]) & " " &  remove_trailing_spaces(CurClient[CL_FIRST_NAME]) & " " & remove_trailing_spaces(CurClient[CL_SURNAME]), CurPatient[PA_NAME]        })
printf(fp, "         %-30.30s              %s\n", {CurClient[CL_ADDRESS1],  CurPatient[PA_SPECIES]       })
printf(fp, "         %-30.30s              %s\n", {CurClient[CL_ADDRESS2],  CurPatient[PA_BREED]      })
printf(fp, "         %-30.30s              %s\n", {CurClient[CL_ADDRESS3],  CurPatient[PA_SEX]          })
printf(fp, "         %-30.30s              %s\n", {CurClient[CL_ADDRESS4],  CurPatient[PA_DOB]         })
printf(fp, "         %-30.30s              %.2f Kg\n", {CurClient[CL_POSTCODE], strtoval(CurPatient[PA_WEIGHT])         })
printf(fp, "         %-30.30s\n", {CurClient[CL_TEL1]})

puts(fp,"--------------------------------------------------------------------------------\n")
puts(fp,"                             Weight report\n")
puts(fp,"--------------------------------------------------------------------------------\n")

puts(fp, "Weight table\n------------\n")

min_wt = 1000
max_wt = 0
for i = 1 to length(weight_table) do
        printf(fp, "%-15.15s %10.2f Kg\n", {JdaystodateStr(weight_table[i][2]), weight_table[i][1]})

        if weight_table[i][1] < min_wt then
                min_wt = weight_table[i][1]
        end if
        if weight_table[i][1] > max_wt then
                max_wt = weight_table[i][1]
        end if
end for

puts(fp, "\n")
printf(fp, "The minimum weight was  : %8.2f Kg\n", {min_wt})
printf(fp, "The maximum weight was  : %8.2f Kg\n", {max_wt})

net_gain = weight_table[$][1] - weight_table[1][1]

if net_gain > 0 then
        printf(fp, "The net weight gain over the period is %8.2f Kg\n", {net_gain})
else
        printf(fp, "The net weight loss over the period is %8.2f Kg\n", {-net_gain})
end if

--weight per week
kgpw =  (net_gain / (weight_table[$][2] - weight_table[1][2]) ) * 7
printf(fp, "That's roughly %8.2f kg / week\n", {kgpw})

close(fp)
system_view_file(fname)



--create graph
graph = repeat( repeat(' ', 80) , 30)
gname = sprintf("%d%d.grph", {CurClient[CL_CLIENT_NO], CurPatient[PA_PATIENT_NO]})

--graph[1][1] = 1st date, min wt
--graph[40][80] - last date, max weight
--graph[x][y]
--x - weight
--y - date
--note, at best these can only be an approximation
--will reserve 1st 10 places for weight scale, 
--and bottom 2 places for date scale
--so in actual fact date scale runs from 10 to 80

--calculate date scale
--calc number of days for each slot
days_diff = weight_table[$][2] - weight_table[1][2]
if days_diff <= 0 then
        --only one or less days - can't make a graph from this !
        return 0
end if
days_per_seg = days_diff / 70

kgs_per_seg = (max_wt - min_wt) / 28

--draw graph lines
for i = 1 to 28 do
        graph[i][8] = '|'
end for
graph[29] = repeat('-', 80)

--put a range of weight onto graph
for i = 0 to 3 do
        graph[28 - (i*9)] = seqcpy( sprintf("%6.2f",{ min_wt + (i * (max_wt - min_wt)/3 ) }), graph[28 - (i*9)], 1)        
end for

--date range -- - put a date every 13 chars
for i = 0 to 70 by 13 do
        if i+13 > 80 then exit end if
        graph[30] = seqcpy( JdaystodateStr( weight_table[1][2] + floor(days_per_seg * i) ), graph[30], i+6 )
end for

--fill the graph with measurements
for i = 1 to length(weight_table) do
        gw = 28 - floor( ( (weight_table[i][1] - min_wt) / (max_wt - min_wt) ) * 28 )
        gd = floor(   ( ((weight_table[i][2] - weight_table[1][2]) / days_diff) * 70 ) + 9   )
        
        if gd > 80 then gd = 80 end if
        if gw <= 0 then gw = 1 end if

        graph[gw][gd] = 'o'

end for

fp = open(gname, "w")

puts(fp, repeat('=', 80) & "\n" )
printf(fp, "Graph of weight changes for %s\n", {remove_trailing_spaces(CurPatient[PA_NAME]) } )
puts(fp, repeat('=', 80) & "\n\n\n" )
puts(fp, "Weight (Kgs)\n\n")

for i = 1 to length(graph) do
        puts(fp, graph[i] & "\n")
end for

puts(fp, "\n                                     Date\n")

close(fp)

system_view_file(gname)

if upper(get_yn_box("Print these? (y/n)[n]") ) = 'Y' then
        printers_print_file(fname, PR_GENERAL)
        printers_print_file(gname, PR_GENERAL)
end if

unlink(fname)
unlink(gname)

return 0
end function